/*****************************************************************************/
/*                      WiNRADiO G35DDC EXCALIBUR ULTRA                      */
/*                                SDK DEMO                                   */
/*                        (C) Copyright RADIXON 2017                         */
/*****************************************************************************/

#include "u_devicepanel.h"
#include "u_mainwindow.h"

enum
{
    ID_DDC1_SPECTRUM=wxID_HIGHEST,    
    ID_IF_SPECTRUM
};

BEGIN_EVENT_TABLE(TDevicePanel, wxPanel)
    EVT_SPECTRUM_TUNE(ID_IF_SPECTRUM,TDevicePanel::OnSpectrumTune)
END_EVENT_TABLE()




TDevicePanel::TDevicePanel(TMainWindow *MainWindow,ICohG35DDCDeviceSet *DeviceSet,uint32_t DeviceIndex):m_DDC1FFTOverlappingBuffer(4*FFT_SIZE),
wxPanel(MainWindow,wxID_ANY,wxPoint(0,0),wxSize(100,100),wxCLIP_CHILDREN|wxBORDER_NONE)
{
 G35DDC_DEVICE_INFO DeviceInfo;
 wxString s;
 wxSizer *MainSizer,*Sizer;

    m_MainWindow=MainWindow;

    m_DeviceSet=DeviceSet;
    m_DeviceIndex=DeviceIndex;

    m_DDC1FFTOverlappingBuffer.SetSize(FFT_SIZE);    

    GetNormalizedWindowCoeffs(m_DDC1FFTCoeffs,FFT_SIZE,1);

    GetNormalizedWindowCoeffs(m_IFFFTCoeffs,FFT_SIZE,1/32768.0);
    m_DeviceSet->GetSpectrumCompensation(m_DeviceIndex,50000000/2,50000000,m_IFFFTCompensation,FFT_SIZE/2);

    MainSizer=new wxBoxSizer(wxVERTICAL);
    SetSizer(MainSizer);


    Sizer=new wxBoxSizer(wxHORIZONTAL);
    MainSizer->Add(Sizer,0,wxEXPAND|wxTOP|wxBOTTOM,4);

    DeviceSet->GetDeviceInfo(DeviceIndex,&DeviceInfo);

    s.sprintf(wxT("Device[%u] - %s"),DeviceIndex,wxString(DeviceInfo.SerialNumber,wxConvLocal).c_str());

    m_LSerNum=new wxStaticText(this,wxID_ANY,s);
    Sizer->Add(m_LSerNum,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddStretchSpacer();

    m_DDC1Spectrum=new TSpectrum(this,ID_DDC1_SPECTRUM);
    MainSizer->Add(m_DDC1Spectrum,1,wxEXPAND);

    m_DDC1Spectrum->SetBand(SpectrumBand_None,false);

    s.sprintf(wxT("DDC1 spectrum / device %u"),m_DeviceIndex);

    m_DDC1Spectrum->SetText(s);

    Sizer->AddSpacer(4);


    m_IFSpectrum=new TSpectrum(this,ID_IF_SPECTRUM);
    MainSizer->Add(m_IFSpectrum,1,wxEXPAND);

    m_IFSpectrum->SetRange(50000000);
    m_IFSpectrum->SetCenterFrequency(50000000/2);
    m_IFSpectrum->SetBand(SpectrumBand_BothSides,false);

    s.sprintf(wxT("IF spectrum / device %u"),m_DeviceIndex);

    m_IFSpectrum->SetText(s);

    UpdateDDC1();
    UpdateDDC1Frequency();

    UpdateControls();
}


TDevicePanel::~TDevicePanel(void)
{
}


void TDevicePanel::UpdateControls(void)
{
 BOOL Power;
 bool e;

    m_DeviceSet->GetPower(&Power);

    e=Power!=FALSE;

    m_IFSpectrum->Enable(e);
    m_DDC1Spectrum->Enable(e);
}

void TDevicePanel::UpdateDDC1FFTCompensation(void)
{
 uint32_t Frequency;

    m_DeviceSet->GetDDC1Frequency(&Frequency);

    m_DDC1FFTMutex.Lock();

    m_DeviceSet->GetSpectrumCompensation(m_DeviceIndex,Frequency,m_DDC1Info.SampleRate,m_DDC1FFTCompensation,FFT_SIZE);

    m_DDC1FFTMutex.Unlock();
}

void TDevicePanel::UpdateDDC1(void)
{
    m_DeviceSet->GetDDC1(NULL,&m_DDC1Info);

    m_DDC1Spectrum->SetRange(m_DDC1Info.Bandwidth);

    m_IFSpectrum->SetBandWidth(m_DDC1Info.Bandwidth);

    m_DDC1FFTOverlappingBuffer.Reset();
}

void TDevicePanel::UpdateDDC1Frequency(void)
{
 uint32_t Frequency;

    m_DeviceSet->GetDDC1Frequency(&Frequency);

    m_DDC1Spectrum->SetCenterFrequency(Frequency);

    m_IFSpectrum->SetBandFrequency(Frequency);

    UpdateDDC1FFTCompensation();
}

void __stdcall TDevicePanel::DDC1StreamCallback(CONST VOID *Buffer,UINT32 NumberOfSamples,UINT32 BitsPerSample)
{
 const float *OverlBuffer;
 float *FFTBuffer,*FFTCoeffs,m,TempI,TempQ;
 uint32_t i;

    if(BitsPerSample==16)
    {
        m_DDC1FFTOverlappingBuffer.AddSamples_16((int16_t*)Buffer,NumberOfSamples);
    }
    else
    {
        m_DDC1FFTOverlappingBuffer.AddSamples_32((int32_t*)Buffer,NumberOfSamples);
    }

    OverlBuffer=m_DDC1FFTOverlappingBuffer.Get();

    if(OverlBuffer)
    {
        FFTBuffer=m_DDC1FFTBuffer;
        FFTCoeffs=m_DDC1FFTCoeffs;

        for(i=0;i<FFT_SIZE;i++)
        {
            FFTBuffer[i*2]=OverlBuffer[i*2]*FFTCoeffs[i];
            FFTBuffer[i*2+1]=OverlBuffer[i*2+1]*FFTCoeffs[i];
        }

        FFT(FFTBuffer,FFT_SIZE);

        //swap lower and upper half of the FFT result        
        for(i=0;i<FFT_SIZE/2;i++)
        {
            TempI=FFTBuffer[i*2];
            TempQ=FFTBuffer[i*2+1];

            FFTBuffer[i*2]=FFTBuffer[(i+FFT_SIZE/2)*2];
            FFTBuffer[i*2+1]=FFTBuffer[(i+FFT_SIZE/2)*2+1];

            FFTBuffer[(i+FFT_SIZE/2)*2]=TempI;
            FFTBuffer[(i+FFT_SIZE/2)*2+1]=TempQ;
        }

        m_DDC1FFTMutex.Lock();

        for(i=0;i<FFT_SIZE;i++)
        {
            m=FFTBuffer[i*2]*FFTBuffer[i*2]+FFTBuffer[i*2+1]*FFTBuffer[i*2+1];

            if(m>0)
            {
                FFTBuffer[i]=10*log10(m);
            }
            else
            {
                FFTBuffer[i]=-180;
            }

            FFTBuffer[i]+=m_DDC1FFTCompensation[i];
        }

        m_DDC1FFTMutex.Unlock();

        m_DDC1Spectrum->SetData(FFTBuffer,FFT_SIZE,m_DDC1Info.SampleRate,(m_DDC1Info.SampleRate-m_DDC1Info.Bandwidth)*0.5);
    }
}

void __stdcall TDevicePanel::IFCallback(CONST SHORT *Buffer,UINT32 NumberOfSamples,WORD MaxADCAmplitude,UINT32 ADCSamplingRate)
{
 uint32_t i;
 float m;

    for(i=0;i<FFT_SIZE;i++)
    {
        m_IFFFTBuffer[i*2]=Buffer[i]*m_IFFFTCoeffs[i];
        m_IFFFTBuffer[i*2+1]=0;
    }

    FFT(m_IFFFTBuffer,FFT_SIZE);

    for(i=0;i<FFT_SIZE/2;i++)
    {
        m=m_IFFFTBuffer[i*2]*m_IFFFTBuffer[i*2]+m_IFFFTBuffer[i*2+1]*m_IFFFTBuffer[i*2+1];

        if(m>0)
        {
            m_IFFFTBuffer[i]=10*log10(m);
        }
        else
        {
            m_IFFFTBuffer[i]=-180;
        }

        m_IFFFTBuffer[i]+=m_IFFFTCompensation[i];
    }

    m_IFSpectrum->SetData(m_IFFFTBuffer,FFT_SIZE/2,50000000,0);
}

void TDevicePanel::OnSpectrumTune(TSpectrumTuneEvent &Event)
{
    m_MainWindow->SetDDC1Frequency((uint32_t)Event.GetFrequency());
}
