/*****************************************************************************/
/*                      WiNRADiO G35DDC EXCALIBUR ULTRA                      */
/*                                SDK DEMO                                   */
/*           Device set selection and "No device set found" dialogs          */
/*                        (C) Copyright RADIXON 2017                         */
/*****************************************************************************/

#include "u_devicesetselector.h"
#include "u_defs.h"
#include "u_global.h"

extern G3XDDCAPI_CREATE_INSTANCE CreateInstance;

TNoDeviceSetFoundWindow::TNoDeviceSetFoundWindow(void):wxDialog(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition)
{
 wxStaticText *t;
 wxSizer *Sizer,*ButtonSizer;
        
    SetFont(*Global.m_Font);

    Sizer=new wxBoxSizer(wxVERTICAL);       

    t=new wxStaticText(this,wxID_ANY,wxT("No coherent G35DDC device set has been found.\n\n")
                                  wxT("This may mean that either no receiver is connected,\n")
                                  wxT("or it is not powered, or its driver was not\n")
                                  wxT("loaded in time before this application was executed.\n\n"),                                  
                    wxDefaultPosition,wxDefaultSize,wxALIGN_CENTRE);
   

    Sizer->Add(t,1,wxALIGN_TOP|wxALIGN_CENTRE|wxTOP,20);
    
    ButtonSizer=new wxBoxSizer(wxHORIZONTAL);

    Sizer->Add(ButtonSizer,0,wxALIGN_CENTER|wxALIGN_TOP|wxBOTTOM,20);

    m_BTryAgain=new wxButton(this,wxID_RETRY,wxT("Try again"),wxDefaultPosition,wxSize(65,23));
    ButtonSizer->Add(m_BTryAgain,0,wxALIGN_CENTER);
    
    ButtonSizer->AddSpacer(4);

    m_BDemo=new wxButton(this,wxID_DEFAULT,wxT("Run in demo mode"),wxDefaultPosition,wxSize(110,23));
    ButtonSizer->Add(m_BDemo,0,wxALIGN_CENTER);

    ButtonSizer->AddSpacer(4);

    m_BCancel=new wxButton(this,wxID_CANCEL,wxT("Cancel"),wxDefaultPosition,wxSize(65,23));
    ButtonSizer->Add(m_BCancel,0,wxALIGN_CENTER);
    
    SetSizer(Sizer);

    SetClientSize(300,153);

    Layout();    

    CenterOnScreen();
}

TNoDeviceSetFoundWindow::~TNoDeviceSetFoundWindow(void)
{
}

void TNoDeviceSetFoundWindow::OnTryAgain(wxCommandEvent &Event)
{
    EndModal(wxID_RETRY);
}

void TNoDeviceSetFoundWindow::OnDemo(wxCommandEvent &Event)
{
    EndModal(wxID_DEFAULT);
}

BEGIN_EVENT_TABLE(TNoDeviceSetFoundWindow, wxDialog)
    EVT_BUTTON(wxID_RETRY,TNoDeviceSetFoundWindow::OnTryAgain)
    EVT_BUTTON(wxID_DEFAULT,TNoDeviceSetFoundWindow::OnDemo)
END_EVENT_TABLE()

TDeviceSetSelector::TDeviceSetSelector(ICohG35DDCDeviceSetEnumerator *Enumerator):wxDialog(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition)
{
 uint32_t i,DeviceCount,j;
 G35DDC_DEVICE_INFO *DeviceInfos;
 wxString *list,s;
 wxPanel *p;
    
    SetFont(*Global.m_Font);

    p=new wxPanel(this,0,0,100,100);

    Layout();
   
    list=new wxString[Enumerator->GetDeviceSetCount()];

    for(i=0;i<Enumerator->GetDeviceSetCount();i++)
    {
        DeviceCount=0;
        Enumerator->GetDeviceSetInfo(i,NULL,&DeviceCount);

        DeviceInfos=new G35DDC_DEVICE_INFO[DeviceCount];

        Enumerator->GetDeviceSetInfo(i,DeviceInfos,&DeviceCount);

        s=wxString(DeviceInfos[0].SerialNumber,wxConvLocal);

        for(j=1;j<DeviceCount;j++)
        {
            s=s+wxT("/")+wxString(DeviceInfos[j].SerialNumber,wxConvLocal);
        }

        list[i]=s;

        delete[] DeviceInfos;
    }

    m_RadioBox=new wxRadioBox(p,wxID_ANY,wxT("Select device set to open"),wxPoint(20,20),wxSize(130,Enumerator->GetDeviceSetCount()*20+25),Enumerator->GetDeviceSetCount(),list,0,wxVERTICAL);

    delete[] list;

    m_BOK=new wxButton(p,wxID_OK,wxT("OK"),wxPoint(165,20),wxSize(70,23));

    m_BCancel=new wxButton(p,wxID_CANCEL,wxT("Cancel"),wxPoint(165,50),wxSize(70,23));
    
    m_DeviceSetIndex=m_RadioBox->GetSelection();

    SetClientSize(m_BOK->GetRect().GetRight()+20,m_RadioBox->GetRect().GetBottom()+20);

    CenterOnScreen();    
}

TDeviceSetSelector::~TDeviceSetSelector(void)
{
}

void TDeviceSetSelector::OnOk(wxCommandEvent &Event)
{
    m_DeviceSetIndex=m_RadioBox->GetSelection();
    EndModal(wxID_OK);
}

BEGIN_EVENT_TABLE(TDeviceSetSelector, wxDialog)
    EVT_BUTTON(wxID_OK,TDeviceSetSelector::OnOk)
END_EVENT_TABLE()


ICohG35DDCDeviceSet* TDeviceSetSelector::SelectDeviceSet(void)
{
 ICohG35DDCDeviceSetEnumerator *Enumerator;
 TNoDeviceSetFoundWindow *NoDeviceSet;
 ICohG35DDCDeviceSet *DeviceSet;
 TDeviceSetSelector *Selector;
 uint32_t Count,DeviceCount;
 int r;
 G35DDC_DEVICE_INFO *DeviceInfos;

    if(!CreateInstance(G35DDC_CLASS_ID_COH_DEVICE_SET_ENUMERATOR,(void**)&Enumerator))
    {
        wxMessageBox(wxT("Failed to create device set enumerator object!"),APP_TITLE,wxICON_ERROR);
        return NULL;
    }

    if(!CreateInstance(G35DDC_CLASS_ID_COH_DEVICE_SET,(void**)&DeviceSet))
    {
        wxMessageBox(wxT("Failed to create device set object!"),APP_TITLE,wxICON_ERROR);
        Enumerator->Release();
        return NULL;
    }

    for(;;)
    {
        Count=Enumerator->GetDeviceSetCount();

        if(Count==0) //no device set has been found
        {
            NoDeviceSet=new TNoDeviceSetFoundWindow;

            r=NoDeviceSet->ShowModal();

            delete NoDeviceSet;

            if(r==wxID_RETRY)
            {
                Enumerator->Enumerate();
            }
            else if(r==wxID_DEFAULT)
            {
                if(DeviceSet->Open(G35DDC_OPEN_DEMO_SET,3))
                {
                    Enumerator->Release();
                    return DeviceSet;
                }

                wxMessageBox(wxT("Failed to open demo device set!"),APP_TITLE,wxICON_ERROR);
                break;
            }
            else
            {
                break;             
            }
        }
        else if(Count==1) //single device set has been found
        {
            DeviceCount=0;
            Enumerator->GetDeviceSetInfo(0,NULL,&DeviceCount);
            
            DeviceInfos=new G35DDC_DEVICE_INFO[DeviceCount];

            Enumerator->GetDeviceSetInfo(0,DeviceInfos,&DeviceCount);

            if(DeviceSet->Open(DeviceInfos,DeviceCount))
            {
                //device set has been open
                delete[] DeviceInfos;
                Enumerator->Release();
                return DeviceSet;
            }

            delete[] DeviceInfos;

            wxMessageBox(wxT("Failed to open device set!"),APP_TITLE,wxICON_ERROR);
            break;
        }
        else
        {
            //more than one device set has been detected
            Selector=new TDeviceSetSelector(Enumerator);

            if(Selector->ShowModal()!=wxID_OK)
            {
                delete Selector;
                break;
            }

            r=Selector->GetSelectedDeviceSetIndex();

            delete Selector;

            DeviceCount=0;
            Enumerator->GetDeviceSetInfo(r,NULL,&DeviceCount);
            DeviceInfos=new G35DDC_DEVICE_INFO[DeviceCount];


            Enumerator->GetDeviceSetInfo(r,DeviceInfos,&DeviceCount);

            if(DeviceSet->Open(DeviceInfos,DeviceCount))
            {
                delete[] DeviceInfos;
                Enumerator->Release();
                return DeviceSet;
            }

            delete[] DeviceInfos;
            wxMessageBox(wxT("Failed to open device set!"),APP_TITLE,wxICON_ERROR);
            break;
        }
    }

    Enumerator->Release();
    DeviceSet->Release();

    return NULL;
}