/*****************************************************************************/
/*                      WiNRADiO G35DDC EXCALIBUR ULTRA                      */
/*                                SDK DEMO                                   */
/*                             Main GUI window                               */
/*                        (C) Copyright RADIXON 2017                         */
/*****************************************************************************/

#include "u_mainwindow.h"
#include "u_defs.h"
#include "u_global.h"


enum
{
    ID_DDC1_FREQUENCY=wxID_HIGHEST,    
    ID_DDC1_BW,
    ID_POWER,
    ID_INIT_POWER_ON_TIMER,
    ID_TIMESTAMP_TIMER
};

BEGIN_EVENT_TABLE(TMainWindow, wxFrame)
    EVT_TOGGLEBUTTON(ID_POWER,TMainWindow::OnButtonClick) 
    EVT_COMBOBOX(ID_DDC1_BW,TMainWindow::OnComboBoxChanged)
    EVT_NUMERICEDIT(ID_DDC1_FREQUENCY,TMainWindow::OnNumericEditChanged)
    EVT_TIMER(ID_INIT_POWER_ON_TIMER,TMainWindow::OnTimer)    
    EVT_TIMER(ID_TIMESTAMP_TIMER,TMainWindow::OnTimer)
END_EVENT_TABLE()


TMainWindow::TMainWindow(ICohG35DDCDeviceSet *DeviceSet):wxFrame(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition,wxSize(900,900)),
m_DeviceSet(DeviceSet),
m_InitPowerOnTimer(this,ID_INIT_POWER_ON_TIMER),
m_TimestampTimer(this,ID_TIMESTAMP_TIMER)
{
 uint32_t i,j,k,Count;
 wxString s;    
 wxSizer *MainSizer,*MidSizer,*Sizer,*DeviceSizer,*PhaseSizer;
 uint32_t Frequency;
 G3XDDC_DDC_INFO DDCInfo;
 wxStaticText *Text;

    m_PrevADCCounter=0;
    m_PrevDDC1Counter=0;

    m_DeviceSet->GetDeviceCount(&m_DeviceCount);
   
    m_DeviceInfos=new G35DDC_DEVICE_INFO[m_DeviceCount];
    m_DeviceSet->GetDeviceInfos(m_DeviceInfos,&m_DeviceCount);

    m_DevicePairCount=m_DeviceCount*(m_DeviceCount-1)/2;

    s=APP_TITLE;
    s=s+wxT(" - ")+wxString(m_DeviceInfos[0].SerialNumber,wxConvLocal);

    for(i=1;i<m_DeviceCount;i++)
    {
        s=s+wxT(" / ")+wxString(m_DeviceInfos[i].SerialNumber,wxConvLocal);
    }

    SetFont(*Global.m_Font);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    SetTitle(s);

    MainSizer=new wxBoxSizer(wxVERTICAL);
    SetSizer(MainSizer);

    Sizer=new wxBoxSizer(wxHORIZONTAL);
    MainSizer->Add(Sizer,0,wxEXPAND|wxALIGN_TOP|wxALL,4);

    m_LDDC1Frequency=new wxStaticText(this,wxID_ANY,wxT("DDC1 frequency"));
    Sizer->Add(m_LDDC1Frequency,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);

    m_DeviceSet->GetDDC1Frequency(&Frequency);
    m_EDDC1Frequency=new TFrequencyEdit(this,ID_DDC1_FREQUENCY,Frequency,0,50000000,wxDefaultPosition,wxSize(100,21));
    Sizer->Add(m_EDDC1Frequency,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(8);

    m_LDDC1BW=new wxStaticText(this,wxID_ANY,wxT("DDC1 BW"));
    Sizer->Add(m_LDDC1BW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);

    m_CBDDC1BW=new wxComboBox(this,ID_DDC1_BW,wxEmptyString,wxDefaultPosition,wxSize(75,23),0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    Sizer->Add(m_CBDDC1BW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DeviceSet->GetDDC1Count(&Count);

    for(i=0;i<Count;i++)
    {
        m_DeviceSet->GetDDCInfo(i,&DDCInfo);

        m_CBDDC1BW->Append(wxT(" ")+FormatFrequency(DDCInfo.Bandwidth));  
    }    

    Sizer->AddSpacer(16);

    m_LDDC1Counter=new wxStaticText(this,wxID_ANY,wxT("DDC1 Counter:"));
    Sizer->Add(m_LDDC1Counter,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);

    m_LDDC1CounterValue=new wxStaticText(this,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(80,m_LDDC1Counter->GetSize().GetHeight()));
    Sizer->Add(m_LDDC1CounterValue,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(8);

    m_LADCCounter=new wxStaticText(this,wxID_ANY,wxT("ADC Counter:"));
    Sizer->Add(m_LADCCounter,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);

    m_LADCCounterValue=new wxStaticText(this,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(80,m_LADCCounter->GetSize().GetHeight()));
    Sizer->Add(m_LADCCounterValue,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(8);

    m_LDDC1SampleRate=new wxStaticText(this,wxID_ANY,wxT("DDC1 Samplerate:"));
    Sizer->Add(m_LDDC1SampleRate,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);

    m_LDDC1SampleRateValue=new wxStaticText(this,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(80,m_LDDC1SampleRate->GetSize().GetHeight()));
    Sizer->Add(m_LDDC1SampleRateValue,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(8);

    m_LADCSampleRate=new wxStaticText(this,wxID_ANY,wxT("ADC Samplerate:"));
    Sizer->Add(m_LADCSampleRate,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);

    m_LADCSampleRateValue=new wxStaticText(this,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(80,m_LADCSampleRate->GetSize().GetHeight()));
    Sizer->Add(m_LADCSampleRateValue,0,wxALIGN_LEFT|wxALIGN_CENTRE);


    MidSizer=new wxBoxSizer(wxHORIZONTAL);
    MainSizer->Add(MidSizer,1,wxEXPAND|wxLEFT|wxRIGHT,4);

    DeviceSizer=new wxBoxSizer(wxVERTICAL);
    MidSizer->Add(DeviceSizer,1,wxEXPAND|wxALIGN_LEFT);

    PhaseSizer=new wxBoxSizer(wxVERTICAL);
    MidSizer->Add(PhaseSizer,1,wxEXPAND|wxLEFT,8);

    m_DevicePanels=new TDevicePanel*[m_DeviceCount];

    for(i=0;i<m_DeviceCount;i++)
    {
        m_DevicePanels[i]=new TDevicePanel(this,m_DeviceSet,i);
        DeviceSizer->Add(m_DevicePanels[i],1,wxEXPAND);
    }

    m_Phases=new TPhase*[m_DevicePairCount];

    for(i=0,j=0;j<m_DeviceCount;j++)
    {
        for(k=j+1;k<m_DeviceCount;k++)
        {
            s.sprintf(wxT("Phase %s - %s"),wxString(m_DeviceInfos[j].SerialNumber,wxConvLocal).c_str(),
                wxString(m_DeviceInfos[k].SerialNumber,wxConvLocal).c_str());

            Text=new wxStaticText(this,wxID_ANY,s);
            PhaseSizer->Add(Text,0,wxALIGN_LEFT|wxTOP|wxBOTTOM,4);

            m_Phases[i]=new TPhase(this,wxID_ANY);
            PhaseSizer->Add(m_Phases[i],1,wxEXPAND);

            i++;
        }
    }

    Sizer=new wxBoxSizer(wxHORIZONTAL);

    MainSizer->Add(Sizer,0,wxEXPAND|wxALL,4);

    Sizer->AddStretchSpacer();

    m_BPower=new wxToggleButton(this,ID_POWER,wxT("Power"),wxDefaultPosition,wxSize(65,23));
    Sizer->Add(m_BPower,0,wxALIGN_RIGHT|wxALIGN_CENTRE);
    
    m_DeviceSet->GetDDC1(&i,&m_DDC1Info);    
    m_CBDDC1BW->SetSelection(i);    
    m_DeviceSet->SetCallback(this);

    SetClientSize(1100,850);
    SetMinSize(wxSize(GetSize().GetWidth(),GetSize().GetHeight()));

    Layout();
    Centre();

    m_InitPowerOnTimer.Start(200,true);
}

TMainWindow::~TMainWindow(void)
{    
    m_DeviceSet->Release();

    delete[] m_DeviceInfos;
    delete[] m_DevicePanels;
}


void __stdcall TMainWindow::CohG35DDC_DDC1StreamCallback(ICohG35DDCDeviceSet *DeviceSet,UINT32 DeviceCount,CONST VOID **Buffers,UINT32 NumberOfSamples,UINT32 BitsPerSample)
{
 uint32_t i,j,k;
 float From,To;

    m_DDC1SampleCounter+=NumberOfSamples;

    for(i=0;i<m_DeviceCount;i++)
    {
        m_DevicePanels[i]->DDC1StreamCallback(Buffers[i],NumberOfSamples,BitsPerSample);
    }

    if(BitsPerSample==16)
    {
        for(i=0,j=0;j<m_DeviceCount;j++)
        {
            for(k=j+1;k<m_DeviceCount;k++)
            {
                GetPhase_int16((int16_t*)Buffers[j],(int16_t*)Buffers[k],NumberOfSamples,&From,&To);

                m_Phases[i]->Add(From,To);

                i++;
            }
        }
    }
    else
    {
        for(i=0,j=0;j<m_DeviceCount;j++)
        {
            for(k=j+1;k<m_DeviceCount;k++)
            {
                GetPhase_int32((int32_t*)Buffers[j],(int32_t*)Buffers[k],NumberOfSamples,&From,&To);

                m_Phases[i]->Add(From,To);

                i++;
            }
        }
    }
}

void __stdcall TMainWindow::CohG35DDC_IFCallback(ICohG35DDCDeviceSet *DeviceSet,UINT32 DeviceIndex,CONST SHORT *Buffer,UINT32 NumberOfSamples,WORD MaxADCAmplitude,UINT32 ADCSamplingRate)
{
    m_DevicePanels[DeviceIndex]->IFCallback(Buffer,NumberOfSamples,MaxADCAmplitude,ADCSamplingRate);
}

void __stdcall TMainWindow::CohG35DDC_DDC2StreamCallback(ICohG35DDCDeviceSet *DeviceSet,UINT32 DeviceIndex,CONST FLOAT *Buffer,UINT32 NumberOfSamples)
{
}

void __stdcall TMainWindow::CohG35DDC_DDC2PreprocessedStreamCallback(ICohG35DDCDeviceSet *Device,UINT32 DeviceIndex,CONST FLOAT *Buffer,UINT32 NumberOfSamples,FLOAT SlevelPeak,FLOAT SlevelRMS)
{
}

void __stdcall TMainWindow::CohG35DDC_AudioStreamCallback(ICohG35DDCDeviceSet *Device,UINT32 DeviceIndex,UINT32 Type,CONST FLOAT *Buffer,UINT32 NumberOfSamples)
{
}

void TMainWindow::SetPower(bool Power)
{
 uint32_t i;
 uint32_t SamplesPerBuffer;
 wxBusyCursor *BusyCursor;

    if(Power)
    {
        m_PrevADCCounter=0;
        m_PrevDDC1Counter=0;
        BusyCursor=new wxBusyCursor;        
    }
    
    m_BPower->SetValue(Power);
    m_BPower->Update();

    m_DeviceSet->SetPower(Power);

    for(i=0;i<m_DeviceCount;i++)
    {
        m_DevicePanels[i]->UpdateControls();
    }

    if(Power)
    {
        m_DDC1SampleCounter=0;

        if(m_DDC1Info.Bandwidth<4000000)
        {
            SamplesPerBuffer=(m_DDC1Info.SampleRate*UPDATE_INTERVAL/1000+63)&(~63);
        }
        else
        {
            SamplesPerBuffer=(m_DDC1Info.SampleRate*UPDATE_INTERVAL/1000+1023)&(~1023);
        }

        m_DeviceSet->StartDDC1(SamplesPerBuffer);

        for(i=0;i<m_DeviceCount;i++)
        {
            m_DeviceSet->StartIF(i,UPDATE_INTERVAL);
        }

        m_TimestampTimer.Start(500);

        delete BusyCursor;
    }    
    else
    {
        m_TimestampTimer.Stop();
    }

    m_LDDC1Frequency->Enable(Power);
    m_EDDC1Frequency->Enable(Power);
    m_LDDC1BW->Enable(Power);
    m_CBDDC1BW->Enable(Power);

    m_LDDC1Counter->Enable(Power);    
    m_LDDC1CounterValue->Enable(Power);
    m_LDDC1CounterValue->SetLabel(wxT("N/A"));

    m_LADCCounter->Enable(Power);    
    m_LADCCounterValue->Enable(Power);
    m_LADCCounterValue->SetLabel(wxT("N/A"));

    m_LDDC1SampleRate->Enable(Power);    
    m_LDDC1SampleRateValue->Enable(Power);
    m_LDDC1SampleRateValue->SetLabel(wxT("N/A"));

    m_LADCSampleRate->Enable(Power);    
    m_LADCSampleRateValue->Enable(Power);
    m_LADCSampleRateValue->SetLabel(wxT("N/A"));
}

void TMainWindow::OnButtonClick(wxCommandEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_POWER:
            SetPower(m_BPower->GetValue());
            break;
    }
}

void TMainWindow::SetDDC1Frequency(uint32_t Frequency)
{
 uint32_t i;
 uint32_t MinFrequency,MaxFrequency;
    
    MinFrequency=m_DDC1Info.Bandwidth/2;
    MaxFrequency=50000000-m_DDC1Info.Bandwidth/2;

    if(Frequency<MinFrequency)
    {
        Frequency=MinFrequency;
    }
    else if(Frequency>MaxFrequency)
    {
        Frequency=MaxFrequency;
    }

    m_DeviceSet->SetDDC1Frequency(Frequency);

    m_EDDC1Frequency->Set(Frequency);

    for(i=0;i<m_DeviceCount;i++)
    {
        m_DevicePanels[i]->UpdateDDC1Frequency();
    }
}

void TMainWindow::OnComboBoxChanged(wxCommandEvent &Event)
{
 uint32_t SamplesPerBuffer,i;

    switch(Event.GetId())
    {
        case ID_DDC1_BW:
            m_DeviceSet->StopDDC1();

            m_DeviceSet->SetDDC1(m_CBDDC1BW->GetSelection());
            m_DeviceSet->GetDDC1(NULL,&m_DDC1Info);

            SamplesPerBuffer=(m_DDC1Info.SampleRate*UPDATE_INTERVAL/1000+63)&(~63);
    
            for(i=0;i<m_DeviceCount;i++)
            {
                m_DevicePanels[i]->UpdateDDC1();
            }

            m_PrevADCCounter=0;
            m_PrevDDC1Counter=0;

            m_DeviceSet->StartDDC1(SamplesPerBuffer);
            break;
    }
}

void TMainWindow::OnNumericEditChanged(wxNotifyEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_DDC1_FREQUENCY:
            SetDDC1Frequency((uint32_t)m_EDDC1Frequency->Get());
            break;
    }
}

void TMainWindow::OnTimer(wxTimerEvent& Event)
{
 double DDC1Counter;
 UINT64 ADCCounter;
 wxString s;

    switch(Event.GetId())
    {
        case ID_INIT_POWER_ON_TIMER:
            SetPower(true);
            break;

        case ID_TIMESTAMP_TIMER:
            if(m_DeviceSet->GetTimestampCounters(&DDC1Counter,&ADCCounter))
            {
                if(ADCCounter!=m_PrevADCCounter)
                {
                    s.sprintf(wxT("%.1f"),DDC1Counter);
                    m_LDDC1CounterValue->SetLabel(s);

                    s.sprintf(wxT("%I64u"),ADCCounter);
                    m_LADCCounterValue->SetLabel(s);

                    if(m_PrevADCCounter!=0)
                    {
                        s.sprintf(wxT("%.6f MHz"),(double)(ADCCounter-m_PrevADCCounter)*(1.0/1000000.0));
                        m_LADCSampleRateValue->SetLabel(s);
                    }

                    if(m_PrevDDC1Counter!=0)
                    {
                        s.sprintf(wxT("%.6f MHz"),(DDC1Counter-m_PrevDDC1Counter)*(1.0/1000000.0));
                        m_LDDC1SampleRateValue->SetLabel(s);
                    }

                    m_PrevADCCounter=ADCCounter; 
                    m_PrevDDC1Counter=DDC1Counter;
                }
            }
            break;
    }
}

void TMainWindow::GetPhase_int16(int16_t *Buffer1,int16_t *Buffer2,uint32_t SamplesPerBuffer,float *From,float *To)
{
 uint32_t i;
 float I1,Q1,I2,Q2;
 float p;

    if(SamplesPerBuffer>PHASE_MAX_SAMPLES)
    {
        SamplesPerBuffer=PHASE_MAX_SAMPLES;
    }

    I1=Buffer1[0]*(1.0/32768.0);
    Q1=Buffer1[1]*(1.0/32768.0);
    I2=Buffer2[0]*(1.0/32768.0);
    Q2=Buffer2[1]*(1.0/32768.0);

    p=atan2((double)(I1*Q2-Q1*I2),(double)(I1*I2+Q1*Q2))*180/M_PI;

    *From=*To=p;

    for(i=1;i<SamplesPerBuffer;i++)
    {
        I1=Buffer1[i*2]*(1.0/32768.0);
        Q1=Buffer1[i*2+1]*(1.0/32768.0);
        I2=Buffer2[i*2]*(1.0/32768.0);
        Q2=Buffer2[i*2+1]*(1.0/32768.0);

        p=atan2((double)(I1*Q2-Q1*I2),(double)(I1*I2+Q1*Q2))*180/M_PI;        

        if(p<*From)
        {
            *From=p;
        }
        else if(p>*To)
        {
            *To=p;
        }
    }
}

void TMainWindow::GetPhase_int32(int32_t *Buffer1,int32_t *Buffer2,uint32_t SamplesPerBuffer,float *From,float *To)
{
 uint32_t i;
 float I1,Q1,I2,Q2;
 float p;

    if(SamplesPerBuffer>PHASE_MAX_SAMPLES)
    {
        SamplesPerBuffer=PHASE_MAX_SAMPLES;
    }

    I1=Buffer1[0]*(1.0/2147483648);
    Q1=Buffer1[1]*(1.0/2147483648);
    I2=Buffer2[0]*(1.0/2147483648);
    Q2=Buffer2[1]*(1.0/2147483648);

    p=atan2((double)(I1*Q2-Q1*I2),(double)(I1*I2+Q1*Q2))*180/M_PI;

    *From=*To=p;

    for(i=1;i<SamplesPerBuffer;i++)
    {
        I1=Buffer1[i*2]*(1.0/2147483648);
        Q1=Buffer1[i*2+1]*(1.0/2147483648);
        I2=Buffer2[i*2]*(1.0/2147483648);
        Q2=Buffer2[i*2+1]*(1.0/2147483648);

        p=atan2((double)(I1*Q2-Q1*I2),(double)(I1*I2+Q1*Q2))*180/M_PI;        

        if(p<*From)
        {
            *From=p;
        }
        else if(p>*To)
        {
            *To=p;
        }
    }
}
