/*****************************************************************************/
/*                      WiNRADiO G35DDC EXCALIBUR ULTRA                      */
/*                                SDK DEMO                                   */
/*                             Main GUI window                               */
/*                        (C) Copyright RADIXON 2017                         */
/*****************************************************************************/

#ifndef __U_MAINWINDOW_H__
#define __U_MAINWINDOW_H__

#include <wx/wx.h>
#include <wx/sizer.h>
#include <wx/tglbtn.h>
#include <wx/combobox.h>
#include "G35DDCAPI.h"
#include "u_spectrum.h"
#include "u_frequencyedit.h"
#include "u_devicepanel.h"
#include "u_phase.h"

class TMainWindow:public wxFrame,public ICohG35DDCDeviceSetCallback
{
public:
    void SetDDC1Frequency(uint32_t Frequency);

    TMainWindow(ICohG35DDCDeviceSet *DeviceSet);
    ~TMainWindow(void);
private:    
    ICohG35DDCDeviceSet         *m_DeviceSet;
    uint32_t                    m_DeviceCount;
    G35DDC_DEVICE_INFO          *m_DeviceInfos;

    wxStaticText                *m_LDDC1Frequency;
    TFrequencyEdit              *m_EDDC1Frequency;
    wxStaticText                *m_LDDC1BW;
    wxComboBox                  *m_CBDDC1BW;

    wxStaticText                *m_LDDC1Counter;
    wxStaticText                *m_LDDC1CounterValue;

    wxStaticText                *m_LADCCounter;
    wxStaticText                *m_LADCCounterValue;

    wxStaticText                *m_LDDC1SampleRate;
    wxStaticText                *m_LDDC1SampleRateValue;

    wxStaticText                *m_LADCSampleRate;
    wxStaticText                *m_LADCSampleRateValue;

    TDevicePanel                **m_DevicePanels;

    wxToggleButton              *m_BPower;

    G3XDDC_DDC_INFO             m_DDC1Info;

    wxTimer                     m_InitPowerOnTimer;
    wxTimer                     m_TimestampTimer;

    UINT64                      m_PrevADCCounter;
    double                      m_PrevDDC1Counter;

    uint32_t                    m_DevicePairCount;
    TPhase                      **m_Phases;
    
    uint64_t                    m_DDC1SampleCounter;

    //callbacks
    void __stdcall CohG35DDC_DDC1StreamCallback(ICohG35DDCDeviceSet *DeviceSet,UINT32 DeviceCount,CONST VOID **Buffers,UINT32 NumberOfSamples,UINT32 BitsPerSample);
    void __stdcall CohG35DDC_IFCallback(ICohG35DDCDeviceSet *DeviceSet,UINT32 DeviceIndex,CONST SHORT *Buffer,UINT32 NumberOfSamples,WORD MaxADCAmplitude,UINT32 ADCSamplingRate);
    void __stdcall CohG35DDC_DDC2StreamCallback(ICohG35DDCDeviceSet *DeviceSet,UINT32 DeviceIndex,CONST FLOAT *Buffer,UINT32 NumberOfSamples);
    void __stdcall CohG35DDC_DDC2PreprocessedStreamCallback(ICohG35DDCDeviceSet *Device,UINT32 DeviceIndex,CONST FLOAT *Buffer,UINT32 NumberOfSamples,FLOAT SlevelPeak,FLOAT SlevelRMS);
    void __stdcall CohG35DDC_AudioStreamCallback(ICohG35DDCDeviceSet *Device,UINT32 DeviceIndex,UINT32 Type,CONST FLOAT *Buffer,UINT32 NumberOfSamples);


    void SetPower(bool Power);

    void OnButtonClick(wxCommandEvent &Event);
    void OnComboBoxChanged(wxCommandEvent &Event);
    void OnNumericEditChanged(wxNotifyEvent &Event);
    void OnTimer(wxTimerEvent& Eevnt);

    void GetPhase_int16(int16_t *Buffer1,int16_t *Buffer2,uint32_t SamplesPerBuffer,float *From,float *To);
    void GetPhase_int32(int32_t *Buffer1,int32_t *Buffer2,uint32_t SamplesPerBuffer,float *From,float *To);

    DECLARE_EVENT_TABLE()
};

#endif

