/*****************************************************************************/
/*                      WiNRADiO G35DDC EXCALIBUR ULTRA                      */
/*                                SDK DEMO                                   */
/*                      Simple frequency spectrum panel                      */
/*                        (C) Copyright RADIXON 2017                         */
/*****************************************************************************/

#ifndef __U_SPECTRUM_H__
#define __U_SPECTRUM_H__

#include <wx/wx.h>
#include "u_types.h"

BEGIN_DECLARE_EVENT_TYPES()
DECLARE_EVENT_TYPE(wxEVT_COMMAND_SPECTRUM_TUNE,5001)
END_DECLARE_EVENT_TYPES()

class TSpectrumTuneEvent:public wxNotifyEvent
{
public:
    inline int64_t GetFrequency(void) const
    {
        return m_Frequency;
    }

    inline void SetFrequency(int64_t Frequency)
    {
        m_Frequency=Frequency;
    }

    TSpectrumTuneEvent(const TSpectrumTuneEvent &Event);
    TSpectrumTuneEvent(int Id=0,int64_t Frequency=0);

	virtual wxEvent* Clone(void) const;

	DECLARE_DYNAMIC_CLASS_NO_ASSIGN(TSpectrumTuneEvent)
private:

    int64_t   m_Frequency;
};

typedef void (wxEvtHandler::*TSpectrumTuneEventFunction)(TSpectrumTuneEvent&);

#define EVT_SPECTRUM_TUNE(id, fn)                                    \
	DECLARE_EVENT_TABLE_ENTRY(wxEVT_COMMAND_SPECTRUM_TUNE, id, -1,   \
	(wxObjectEventFunction)(wxEventFunction)                         \
	(TSpectrumTuneEventFunction) & fn, (wxObject*) NULL ),


typedef enum
{
    SpectrumBand_None=0,
    SpectrumBand_BothSides,
    SpectrumBand_LowerSide,
    SpectrumBand_UpperSide
} TSpectrumBand;

class TSpectrum:public wxPanel
{
public:
    inline TSpectrumBand GetBand(void) const
    {
        return m_Band;
    }

    void SetBand(TSpectrumBand Band,bool Relative);
    void SetBandFrequency(int64_t Frequency);

    inline int64_t GetBandFrequency(void) const
    {
        return m_BandFrequency;
    }

    void SetBandWidth(uint32_t Width);

    inline uint32_t GetBandWidth(void) const
    {
        return m_BandWidth;
    }

    void Clear(void);

    inline int64_t GetCenterFrequency(void) const
    {
        return m_CenterFrequency;
    }
    void SetCenterFrequency(int64_t Frequency);

    inline int GetRange(void) const
    {
        return m_Range;
    }
    void SetRange(int Range);

    inline int GetMinLevel(void) const
    {
        return m_MinLevel;
    }
    void SetMinLevel(int Level);

    void SetData(float *Data,uint32_t Count,double Range,double Offset);
    void SetText(const wxString &Text);

    virtual bool Enable(bool enable);

    TSpectrum(wxWindow *Parent,wxWindowID id,const wxPoint &Position=wxDefaultPosition,const wxSize &Size=wxDefaultSize);
    ~TSpectrum(void);
protected:
private:
    int                 m_Range;    
    int64_t             m_CenterFrequency;
    int                 m_MinLevel;
    wxString            m_Text;

    wxRect              m_GraphRect;

    TSpectrumBand       m_Band;
    int64_t             m_BandFrequency;
    int                 m_BandWidth;
    bool                m_BandRelative;
    bool                m_BandFrequencyCursor;

    double              m_Frequency2Pixel;
    double              m_Pixel2Frequency;
    double              m_FrequencyStep;

    double              m_Level2Pixel;
    double              m_Pixel2Level;
    int                 m_LevelStep;
    wxMutex             m_Mutex;

    float               *m_Data;
    uint32_t            m_DataCount;
    bool                m_DataChanged;
    bool                m_DataValid;
    double              m_DataRange;
    double              m_DataOffset;

    bool                m_DisplayDataValid;
    wxPoint             *m_DisplayData;
    uint32_t            m_DisplayDataCount;
    bool                m_TuneWhenMouseMove;

    void OnPaintEvent(wxPaintEvent &Event);
    void OnSizeEvent(wxSizeEvent &Event);
    void OnEraseBackgroundEvent(wxEraseEvent &Event);
    void OnData(wxNotifyEvent &Event);
    void OnMouseDown(wxMouseEvent &Event);
    void OnMouseUp(wxMouseEvent &Event);
    void OnMouseMove(wxMouseEvent &Event);
    void OnMouseCaptureLost(wxMouseCaptureLostEvent &Event);


    void UpdateParams(void);
    void UpdateDisplayData(void);
    void Tune(wxPoint &Pos);

    DECLARE_EVENT_TABLE()
};

#endif
